/***************************************************************************
 * File:          channel.h
 * Description:   Data structure for IDE channel
 * Copyright (c)  2000-2003 HighPoint Technologies, Inc. All rights reserved
 ***************************************************************************/


#ifndef _CHANNEL_H_
#define _CHANNEL_H_

/***************************************************************************
 * Description:  Scatter-gather table
 ***************************************************************************/

#pragma pack(1)
typedef struct _SCAT_GATH
{
    BUS_ADDR      dSgAddress;        /* Physical address of main memory */
    USHORT        wSgSize;           /* length of this block */
    USHORT        wSgFlag;           /* 0 following next SG, 0x8000 last SG */
} SCAT_GATH, FAR *FPSCAT_GATH;
typedef struct _SCAT_GATH32
{
    ULONG         dSgAddress;
    USHORT        wSgSize;
    USHORT        wSgFlag;
} SCAT_GATH32, FAR *FPSCAT_GATH32;
#pragma pack()

#define SG_FLAG_SKIP        0x4000
#define SG_FLAG_EOT         0x8000  /* End flag of SG list */

/***************************************************************************
 * Description: Channel Table
 ***************************************************************************/

typedef struct _Channel {
    /*
     * hardware information. read only for channel layer after initialization
     */
    PIDE_REGISTERS_1 BaseIoAddress1;  /* IDE Base Port Address */
    PIDE_REGISTERS_2 BaseIoAddress2;  /* IDE Control Port Address */
    PUCHAR           BMI;             /* IDE DMA extended Port Addresss */
    union _uChannel {
		PUCHAR       _MiscControlAddr;
		#define MiscControlAddr u._MiscControlAddr
		UCHAR no_use;
	} u;

    PChipInstance    pChipInstance;
    PDevice          pDevice[2];      /* point to the exsiting devices */

    /*
     * Resource, read only
     */
    FPSCAT_GATH      pSgTable;        /* point to SG table of this channel */
    ULONG_PTR        SgPhysicalAddr;  /* physcial address of SG table */


    /*
     * The data for command executing, w/r by channel layer
     */
    /* flags */
    USHORT cf_80pin_cable: 1;
    USHORT cf_cable_check: 1;
    USHORT cf_do_dma: 1;
    USHORT cf_timer_callback: 1;
    USHORT cf_timer_send_command: 1;
	USHORT cf_resetting: 1;
	USHORT cf_hardware_tcq: 1; /* use hardware polling */

	UCHAR            bMaxMode;        /* maximum transfer mode; valid if not zero */
    UCHAR            bChRetry;		  /* retry count */

    PDevice          pChWorkDev;      /* current Working device */
    PCommand         CurrentCmd;

#ifdef CHECK_HARDWARE_TIMEOUT
	ULONG            Timer;
#endif


} Channel;

/***************************************************************************
 * Fuction Prototype
 ***************************************************************************/

/*
 * timer support provided by platform dependent layer 
 *
 *  ULONG AddTimerCall(void (*func)(void *), void * arg, ULONG timeout);
 *  - Return non-zero timer id.
 *
 *  void  DelTimerCall(ULONG timer);
 *  - Cancel a timer. Call only when the timer routine is not executed.
 */
ULONG AddTimerCall(_VBUS_ARG void (*func)(void *), void * arg, ULONG timeout);
void  DelTimerCall(_VBUS_ARG ULONG timer);

/*
 * public function
 */
void fChannelSendCommand(_VBUS_ARG PDevice pDev, PCommand pCmd);
BOOLEAN fChannelInterrupt(_VBUS_ARG PChannel pChan);
BOOLEAN HPTLIBAPI fChipInterrupt(PChipInstance pCI);
BOOLEAN fPioInterrupt(PDevice pDevice);
UINT CheckCmqQueServiceRequest(PDevice pDev);

/*
 * privacy function (called in channel layer)
 */
UCHAR __fWaitOnBusy(PIDE_REGISTERS_2 BaseIoAddress);
UCHAR fWaitOnBaseBusy(PIDE_REGISTERS_1 BaseIoAddress) ;
UCHAR fWaitForDrq(PIDE_REGISTERS_2 BaseIoAddress);

/***************************************************************************
 * macro 
 ***************************************************************************/

#define mDeviceRemoved(stat) ((stat & 0x7e)== 0x7e || (stat & 0x40) == 0)

#endif
