/***************************************************************************
 * File:          device.h
 * Description:   Data structure for Device
 * Copyright (c)  2000-2003 HighPoint Technologies, Inc. All rights reserved
 ***************************************************************************/


#ifndef _HPT_DEVICE_H_
#define _HPT_DEVICE_H_

/***************************************************************************
 * Description:  Device Table
 ***************************************************************************/

typedef struct _Device {
	UINT df_lba                  :1;   /* Use LBA mode on drive */
	UINT df_dma                  :1;   /* Use DMA on the drive */
	UINT df_multiple             :1;   /* Use Multi-Block commands */
	UINT df_ultra                :1;   /* Support ultra DMA */
	UINT df_harddisk             :1;   /* This is a non-removable disk */
	UINT df_atapi                :1;   /* This is ATAPI device */
#define df_intr_drq df_multiple
	UINT df_removable_drive      :1;   /* ATA removable device (IDE ZIP) */
	UINT df_device_lockable      :1;   /* This devices is lockable */
	UINT df_cdrom_device         :1;   /* This is a CDROM device */
	UINT df_ls120                :1;   /* This is a LS-120 */
	UINT df_tape_device          :1;   /* TAPE device */
	UINT df_atapi_changer        :1;   /* Indicates atapi 2.5 changer present. */
	UINT df_media_status_enabled :1;   /* This device receive :1DA    command */
	UINT df_media_changed        :1;   /* removable media changed. */
	UINT df_support_48bit_lba    :1;   
	
	UINT df_need_switch          :1;   /* This disk need fix HPT370 bug */
	
	/* user requests */
	UINT df_user_mode_set        :1;   /* bDeUserSelectMode is valid */
	
	UINT df_on_line              :1;   /* otherwise: dead */
	UINT df_reduce_mode          :1;   /* reduce mode */
	UINT df_sata                 :1;   /* is a serial ATA drive */
	UINT df_with_601             :1;   /* with HPT601 IC */

    PChannel pChannel;         /* The channel this device attached */

    UCHAR   bDeUnitId;         /* Device ID: 0xA0 or 0x B0 */
    UCHAR   bDeModeSetting;    /* Current Data Transfer mode: 0-4 PIO 0-4 */
	UCHAR   wResetCount;       /* how many times device got reset */
	UCHAR   BusyCount;

    /*
     * device mode
     */
    UCHAR   bDeBestPIO;           /* Best PIO mode of this device */
    UCHAR   bDeBestDMA;           /* Best MW DMA mode of this device */
    UCHAR   bDeBestUDMA;          /* Best Ultra DMA mode of this device */
    UCHAR   bMaxShowMode;

    UCHAR   bDeUsable_Mode;       /* actual maximum data transfer mode */
	UCHAR   bDeUserSelectMode;
    UCHAR   DeviceID;
    UCHAR   bSubDevices;          /* how many sub-device it has */

    USHORT  wWordsPerIntr;        /* Word count per interrupt */
    UCHAR   bDeReadCmd;           /* Read or Read Multiple */
    UCHAR   bDeWriteCmd;          /* Write or Write Multiple */

    USHORT  wDeRealHeadXsect;     /* Disk physical head */
    UCHAR   bDeRealSector;        /* Disk physical sector per track */
    UCHAR   bDeRealHeader;        /* Disk physical head */

    ULONG   dDeRealCapacity;      /* 1 less than number of blocks! */
    ULONG   dDeHiddenLba;         /* hidden LBA from beginning */    

    /*
     *  'soft' command queue
     */
	PCommand QueuedJob;
	UINT     QueueLength;
	LBA_T    HeadPosition;



    IDENTIFY_DATA2  IdentifyData;
	USHORT pad2; /* padding to DWORD size */
} Device;

#ifndef IS_SATA_DEVICE
#define IS_SATA_DEVICE(pDev) (pDev->df_sata)
#endif

/***************************************************************************
 * Description:  Function Protocal
 ***************************************************************************/

int fNonIoAtaCmd(PDevice pDevice, UCHAR CommandCode);
UCHAR fSetAtaCmd(PDevice pDev, UCHAR cmd);
UINT fGetMediaStatus(PDevice pDev);
void fDeSetDevice(PDevice pDev);
void HPTLIBAPI fDeSelectMode(PDevice pDev, UCHAR NewMode);
int IdeHardReset(PIDE_REGISTERS_1 DataPort, PIDE_REGISTERS_2 ControlPort);
int HPTLIBAPI fDeReadWrite(PDevice pDev, ULONG Lba, UCHAR Cmd, void *tmpBuffer);
int  fDeScanAllDevices(PChipInstance pChip);
int  fDeScanDevice(PDevice pDevice);

#ifdef SUPPORT_HPT601			
void BeginAccess601(PIDE_REGISTERS_1 IoPort); 
void EndAccess601(PIDE_REGISTERS_1 IoPort);
#endif

/*
 * functions for initialization
 */
void fSetDeviceTable(PDevice pDev, PIDENTIFY_DATA pIdentify);
PBadModeList check_bad_disk(char *ModelNumber);

/***************************************************************************
 * Description:  Macro Define
 ***************************************************************************/

#define mDeDeviceRemoved(bIdeStatus) \
    ((statusByte & 0x7e)== 0x7e || (statusByte & 0x40) == 0))

PCommand FASTCALL GetCommandFromQueue(PDevice pDev);
PCommand FASTCALL PickCommandFromQueue(PDevice pDev);
BOOLEAN FASTCALL PutCommandToQueue(PDevice pDev, PCommand pCmd);

/***************************************************************************
 * Description: Bad Device
 ***************************************************************************/

typedef struct _BadModeList {
    UCHAR      UltraDMAMode;     /* 0xFF not support, others useable mode */
    UCHAR      DMAMode;             /* 0xFF not support, others useable mode */
    UCHAR      PIOMode;             /* 0xFF not support, others useable mode */
    UCHAR      length;             /* length of identify string */
    char      *name;           /* point to identify string */
} BadModeList;

#endif
