/*
 * Rocket100 Controller Device Driver for Linux
 * Copyright (C) 2000-2003 HighPoint Technologies, Inc.
 * proc.c  /proc file system support
 */
#include "global.h"
#include "oslinux.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
/*
 * proc entry
 */
struct proc_dir_entry proc_scsi_hpt3xx = { 
	0, 							/* inode number - ignore. filled by proc_register_dynamic */
	sizeof(PROC_DIR_NAME)-1, 	/* length of name */
	PROC_DIR_NAME,              /* file name */
	S_IFDIR|S_IRUGO|S_IXUGO,	/* */
	2							/* number of links */
};
#endif

/*************************************************************************
 * hpt_set_asc_info
 * Usage:
 *   echo "hpt dbglevel x" > /proc/scsi/hpt374/0
 *        x can be 0, 1, 2, 3
 *************************************************************************/

int hpt_set_asc_info(char *buffer,int length)
{
	return -EINVAL;
}

int hpt_set_info(char *buffer,int length)
{
	int retval;
	if (length >= 4) {
		if (strncmp(buffer,"hpt ",4) == 0) {
			return (retval = hpt_set_asc_info(buffer+4, length-4)) >= 0? retval : -EINVAL;
		}
    }
    return -EINVAL;
}

typedef struct _hpt_GET_INFO {
	char *buffer;
	int buflength;
	int bufoffset;
	int buffillen;
	int filpos;	
} HPT_GET_INFO;

void hpt_copy_mem_info(HPT_GET_INFO *pInfo, char *data, int datalen)
{
	if (pInfo->filpos < pInfo->bufoffset) {	/* Current offset before buffer offset */
		if (pInfo->filpos + datalen <= pInfo->bufoffset) {
			pInfo->filpos += datalen; 		/* Discard if completely before buffer */
			return;
		} else {				/* Partial copy, set to begin */
			data += (pInfo->bufoffset - pInfo->filpos);
			datalen  -= (pInfo->bufoffset - pInfo->filpos);
			pInfo->filpos = pInfo->bufoffset;
		}
	}
	
	pInfo->filpos += datalen;		/* Update current offset */
	if (pInfo->buffillen == pInfo->buflength) /* Buffer full, discard */
		return;
	
	if (pInfo->buflength - pInfo->buffillen < datalen)  /* Overflows buffer ? */
	datalen = pInfo->buflength - pInfo->buffillen;
	
	memcpy(pInfo->buffer + pInfo->buffillen, data, datalen);
	pInfo->buffillen += datalen;
}

#define shortswap(w) ((WORD)((w)>>8 | ((w) & 0xFF)<<8))

static void get_disk_name(char *name, PDevice pDev)
{
	int i;
	for (i = 0; i < 10; i++) 
		((WORD*)name)[i] = shortswap(pDev->IdentifyData.ModelNumber[i]);
	name[20] = '\0';
}

int hpt_copy_info(HPT_GET_INFO *pinfo, char *fmt, ...) 
{
	va_list args;
	char buf[128];
	int len;
  
	va_start(args, fmt);
	len = vsprintf(buf, fmt, args);
	va_end(args);
	hpt_copy_mem_info(pinfo, buf, len);
	return len;
}

static void hpt_copy_disk_info(HPT_GET_INFO *pinfo, PVDevice pVDev)
{
	UINT iChip, iChan, iDev;
	char name[32];
	get_disk_name(name, &pVDev->u.disk);
	
	for(iChip = 0; iChip < pChipEnd-gChipInstances; iChip++)
		for(iChan = 0; iChan < MAX_CHANNEL_PER_CHIP; iChan++)
			for(iDev = 0; iDev < MAX_DEVICE_PER_CHANNEL; iDev++)
	          if(&pVDev->u.disk == gChipInstances[iChip].pChannel[iChan]->pDevice[iDev])
              {
				  hpt_copy_info(pinfo, "%d Channel %d %s %s  %5dMB  %s\n",
					  iChip+1, iChan+1, iDev?"Slave ":"Master", 
					  name, pVDev->VDeviceCapacity>>11, 
					  ((!pVDev->u.disk.df_on_line)? "Disabled" : 
					  ((pVDev->VDeviceType != VD_SPARE)?"Normal  ":"Spare   ")));
				  return;
			  }
}

int hpt_get_info(char *buffer,char **start,off_t offset,int length)
{
	HPT_GET_INFO info;
	UINT iBus,iChip,j,i;
	PDevice pDev;
 
	info.buffer     = buffer;
	info.buflength  = length;
	info.bufoffset  = offset;
	info.filpos     = 0;
	info.buffillen  = 0;

	if (start) *start = buffer;

	hpt_copy_info(&info, "%s Version %s\n\n", DRIVER_NAME, DRIVER_VERSION);

	hpt_copy_info(&info, "Physical device list\n");
	hpt_copy_info(&info, "Controller/Bus/ID  Model                Capacity  Status\n");
	hpt_copy_info(&info, "---------------------------------------------------------\n");
	for(iBus = 0; iBus < MAX_VBUS; iBus++) 			
		for(iChip = 0; iChip < gVBus[iBus].nInstances; iChip++)
			for(j = 0; j < MAX_CHANNEL_PER_CHIP; j++)
				for(i = 0; i < MAX_DEVICE_PER_CHANNEL; i++)
	            {
					pDev=gVBus[iBus].pChipInstance[iChip]->pChannel[j]->pDevice[i];
					if(pDev)
					   hpt_copy_disk_info(&info, Map2pVDevice(pDev));
				}
	
	return info.buffillen;
}

/*******************************************************************************
 * hpt3xx_ProcInfo
 * Parameters:
 *  *buffer: I/O buffer
 *  **start: if inout == FALSE pointer into buffer where user read should start
 *  offset: current offset into a /proc/scsi/hpt37x2/[0...] file
 *  length: length of buffer
 *  hostno: Scsi_Host host_no
 *  inout: TRUE - user is writing; FALSE - user is reading
 * Returns:
 *   Length of data
 *****/
int hpt3xx_ProcInfo(char *buffer,char **start,off_t offset,int length,   
                   int hostno,int inout)
{
	KdPrintI(("hpt3xx_ProcInfo() length %d ha %d offs %d inout %d\n",
		length,hostno,(int)offset,inout));

	if (inout)
		return(hpt_set_info(buffer,length));
	else
		return(hpt_get_info(buffer,start,offset,length));
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
int hpt3xx_ProcInfo26(struct Scsi_Host *host, char *buffer, char **start, 
			off_t offset, int length, int inout)
{
	return hpt3xx_ProcInfo(buffer, start, offset, length, host->host_no, inout);
}
#endif
