/*
 * osheader.h for Linux
 * Copyright (C) HighPoint Technologies, Inc. All rights reserved.
 *
 *   OS header files and definition that provides necessary support
 *   for platform-independent driver code:
 *
 *     1) Basic data types: ULONG, USHORT, UCHAR, ...
 *     2) Hardware I/O functions: InPort, OutPort, ...
 *     3) PCI configuration space access
 *     4) Arithmetic function: LongDiv, LongMul, ...
 *     5) C linrary: ZeroMemory, MemoryCopy
 *     6) Driver related: StallExec, fOsPhysicalAddress, fAsMapLbaToCHS,
 *        fNotifyGUI, VDevice_Ext
 *     7) Debug support: KdPrint, HPT_ASSERT
 *
 *   Platform dependent driver structures and data shall not be defined
 *   in this file except VDevice_Ext.
 */

#ifndef _OSHEADER_H_
#define _OSHEADER_H_

/*************************************************************************
 * compiler, data types
 *************************************************************************/
#define FAR
#ifdef FASTCALL
#undef FASTCALL
#endif
#define HPTLIBAPI __attribute__((regparm(0)))
#define FASTCALL HPTLIBAPI
#define PASCAL HPTLIBAPI


typedef unsigned short USHORT;
typedef unsigned char  UCHAR;
typedef unsigned char *PUCHAR;
typedef unsigned short *PUSHORT;
typedef unsigned char  BOOLEAN;
typedef unsigned short WORD;
typedef unsigned int   UINT, BOOL;
typedef unsigned char  BYTE;
typedef void *PVOID, *LPVOID;
typedef void *ADDRESS;

typedef int  LONG;
typedef unsigned int ULONG, *PULONG, LBA_T;
typedef unsigned int DWORD, *LPDWORD, *PDWORD;
typedef unsigned long ULONG_PTR, UINT_PTR;
typedef unsigned long long BUS_ADDR;

WORD  HPTLIBAPI CPU_TO_LE16(WORD x);  
DWORD HPTLIBAPI CPU_TO_LE32(DWORD x); 
WORD  HPTLIBAPI LE16_TO_CPU(WORD x);  
DWORD HPTLIBAPI LE32_TO_CPU(DWORD x);
WORD  HPTLIBAPI CPU_TO_BE16(WORD x); 
DWORD HPTLIBAPI CPU_TO_BE32(DWORD x);
WORD  HPTLIBAPI BE16_TO_CPU(WORD x); 
DWORD HPTLIBAPI BE32_TO_CPU(DWORD x);
/*************************************************************************
 * arithmetic functions 
 *************************************************************************/
#define LongRShift(x, y) 	(x >> y)
#define LongLShift(x, y)   	(x << y)
#define LongDiv(x, y)      	(x / (UINT)(y))
#define LongRem(x, y)		(x % (UINT)(y))
#define LongMul(x, y)      	(x * y)
/* Minimum and maximum macros */
#define MAX(a, b)	(((a) > (b)) ? (a) : (b))
#define MIN(a, b)	(((a) < (b)) ? (a) : (b))

/*************************************************************************
 * hardware I/O functions                                                *
 *************************************************************************/
BYTE HPTLIBAPI hpt_inp(unsigned port);
WORD HPTLIBAPI hpt_inpw(unsigned port);
DWORD HPTLIBAPI hpt_inpd(unsigned port);
void HPTLIBAPI hpt_outp(unsigned port, BYTE value);
void HPTLIBAPI hpt_outpw(unsigned port, WORD value);
void HPTLIBAPI hpt_outpd(unsigned port, DWORD value);
void HPTLIBAPI hpt_repinsw(unsigned ioport, ADDRESS buffer, WORD count);
void HPTLIBAPI hpt_repoutsw(unsigned ioport, ADDRESS buffer, WORD count);
void HPTLIBAPI hpt_repinsd(unsigned ioport, ADDRESS buffer, WORD count);
void HPTLIBAPI hpt_repoutsd(unsigned ioport, ADDRESS buffer, WORD count);

#define T_IOPORT(x)			((USHORT)(ULONG_PTR)(x))
#define OutDWord(x, y)    hpt_outpd(T_IOPORT(x), (DWORD)(y))
#define InDWord(x)        hpt_inpd(T_IOPORT(x))
#define RepINS(x,y,z)     hpt_repinsw(T_IOPORT(x),(ADDRESS)y,(WORD)z)
#define RepOUTS(x,y,z)    hpt_repoutsw(T_IOPORT(x),(ADDRESS)y,(WORD)z)
#define RepINSD(x,y,z)    hpt_repinsd(T_IOPORT(x),(ADDRESS)y,(WORD)z)
#define RepOUTSD(x,y,z)   hpt_repoutsd(T_IOPORT(x),(ADDRESS)y,(WORD)z)
#define OutPort(x, y)     hpt_outp(T_IOPORT(x),y)  
#define InPort(x)         hpt_inp(T_IOPORT(x))  
#define OutWord(x, y)     hpt_outpw(T_IOPORT(x),y)  
#define InWord(x)         hpt_inpw(T_IOPORT(x))


/*************************************************************************
 * PCI configuration space access
 *************************************************************************/
UCHAR HPTLIBAPI pcicfg_read_byte(UCHAR bus, UCHAR dev, UCHAR func, UCHAR reg);
WORD  HPTLIBAPI pcicfg_read_word (UCHAR bus, UCHAR dev, UCHAR func, UCHAR reg);
DWORD HPTLIBAPI pcicfg_read_dword(UCHAR bus, UCHAR dev, UCHAR func, UCHAR reg);
void HPTLIBAPI pcicfg_write_byte(UCHAR bus, UCHAR dev, UCHAR func, UCHAR reg, UCHAR v);
void HPTLIBAPI pcicfg_write_word (UCHAR bus, UCHAR dev, UCHAR func, UCHAR reg, WORD v);
void HPTLIBAPI pcicfg_write_dword(UCHAR bus, UCHAR dev, UCHAR func, UCHAR reg, DWORD v);

/*************************************************************************
 * C library
 *************************************************************************/
#ifdef NO_LIBC
int HPTLIBAPI os_memcmp(const void *cs, const void *ct, unsigned len);
void os_memcpy(void *to, const void *from, unsigned len);
void HPTLIBAPI os_memset(void *s, char c, unsigned len);
unsigned os_strlen(const char *s);
#define memcmp os_memcmp
#define memcpy os_memcpy
#define memset os_memset
#define strlen os_strlen
#endif
#define ZeroMemory(a, b)  	memset((char *)a, 0, b)
#define MemoryCopy(a,b,c) 	memcpy((char *)(a), (char *)(b), (UINT)(c))
#define StrLen            	strlen
#define farMemoryCopy(a,b,c) memcpy((char *)(a), (char *)(b), (UINT)c)

void * HPTLIBAPI os_kmap_ptr(BUS_ADDR addr);
void HPTLIBAPI os_kunmap_ptr(void *ptr);

/*************************************************************************
 * Driver related
 *************************************************************************/
ULONG_PTR fOsPhysicalAddress(void *addr);
void StallExec(DWORD x);
ULONG fAsMapLbaToCHS(ULONG Lba, WORD sectorXhead, BYTE head);
#define fNotifyGUI(WhatHappen, pVDevice)
#define DECLARE_BUFFER(type, ptr) UCHAR ptr##__buf[512]; type ptr=(type)ptr##__buf

/*************************************************************************
 * Debug support
 *************************************************************************/
#if DBG
/*#define HPT_ASSERT(x) if (!(x)) \
						__asm__ ( "int $3"); \
						else*/
#define HPT_ASSERT(x)
extern int hpt_dbg_level;
extern int os_printk(char *fmt, ...);
#define KdPrint(_x_) os_printk _x_
#define KdPrintI(_x_) do{ if (hpt_dbg_level>0) KdPrint(_x_); }while(0)
#define KdPrintW(_x_) do{ if (hpt_dbg_level>1) KdPrint(_x_); }while(0)
#define KdPrintE(_x_) do{ if (hpt_dbg_level>2) KdPrint(_x_); }while(0)
#else 
#define HPT_ASSERT(x)
#define KdPrint(x) 
#define KdPrintI(x) 
#define KdPrintW(x) 
#define KdPrintE(x) 
#endif

#endif
