/*
 * HPT3xx UDMA/ATA RAID Controller Device Driver for Linux
 * Copyright (C) 2000-2003 HighPoint Technologies, Inc.
 * oslinux.h  Linux headers for platform dependent part.
 */
#ifndef _OSLINUX_H_
#define _OSLINUX_H_

#define hpt_printk(x) do { printk(PROC_DIR_NAME": "); printk x; } while (0)

int HPTLIBAPI os_memcmp(const void *cs, const void *ct, unsigned len) { return memcmp(cs, ct, len); }
void HPTLIBAPI os_memcpy(void *to, const void *from, unsigned len) { memcpy(to, from, len); }
void HPTLIBAPI os_memset(void *s, char c, unsigned len) { memset(s, c, len); }
unsigned HPTLIBAPI os_strlen(const char *s) { return strlen(s); }

extern struct Scsi_Host *hpt_vhost;

typedef struct 
{
	UCHAR		status;     		/* 0 nonbootable; 80h bootable */
	UCHAR      	start_head;
	USHORT     	start_sector;
	UCHAR      	type;
	UCHAR      	end_head;
	USHORT     	end_sector;
	ULONG      	start_abs_sector;
	ULONG      	num_of_sector;
} partition;

typedef struct _INQUIRYDATA {
#ifdef __BIG_ENDIAN_BITFIELD
	UCHAR DeviceTypeQualifier : 3;
	UCHAR DeviceType : 5;
	UCHAR RemovableMedia : 1;
	UCHAR DeviceTypeModifier : 7;
#else 
	UCHAR DeviceType : 5;
	UCHAR DeviceTypeQualifier : 3;
	UCHAR DeviceTypeModifier : 7;
	UCHAR RemovableMedia : 1;
#endif
	UCHAR Versions;
	UCHAR ResponseDataFormat;
	UCHAR AdditionalLength;
	UCHAR Reserved[2];
#ifdef __BIG_ENDIAN_BITFIELD
	UCHAR RelativeAddressing : 1;
	UCHAR Wide32Bit : 1;
	UCHAR Wide16Bit : 1;
	UCHAR Synchronous : 1;
	UCHAR LinkedCommands : 1;
	UCHAR Reserved2 : 1;
	UCHAR CommandQueue : 1;
	UCHAR SoftReset : 1;
#else 
	UCHAR SoftReset : 1;
	UCHAR CommandQueue : 1;
	UCHAR Reserved2 : 1;
	UCHAR LinkedCommands : 1;
	UCHAR Synchronous : 1;
	UCHAR Wide16Bit : 1;
	UCHAR Wide32Bit : 1;
	UCHAR RelativeAddressing : 1;
#endif
	UCHAR VendorId[8];
	UCHAR ProductId[16];
	UCHAR ProductRevisionLevel[4];
	UCHAR VendorSpecific[20];
	UCHAR Reserved3[40];
} INQUIRYDATA, *PINQUIRYDATA;

/* Read Capacity Data - returned in Big Endian format */
typedef struct _READ_CAPACITY_DATA {
	ULONG LogicalBlockAddress;
	ULONG BytesPerBlock;
} READ_CAPACITY_DATA, *PREAD_CAPACITY_DATA;

struct hpt3xx_HOSTDATA {
	Scsi_Cmnd *pending_Q;
		/* pending commands to issue */
	Scsi_Cmnd *current_Q;
		/* current commands on the bus */
	UCHAR commands;		
	    /* number of current commands,this way only allow commands==32 */
	UCHAR status;
		/* host lock */
	UCHAR irq[MAX_CHIPS];
	struct pci_dev *pci_dev;
};
/* access macros for hostdata */
#define HOSTDATA(shpnt)		((struct hpt3xx_HOSTDATA *) &hpt_vhost->hostdata)
#define PENDING_Q           (HOSTDATA(shpnt)->pending_Q)
#define CURRENT_Q		    (HOSTDATA(shpnt)->current_Q)
#define CURRCMDs            (HOSTDATA(shpnt)->commands)

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
#define sc_host(sc)    sc->device->host
#define sc_channel(sc) sc->device->channel
#define sc_target(sc)  sc->device->id
#define sc_lun(sc)     sc->device->lun
#define sc_flags(sc)     sc->eh_eflags
#else 
#define sc_host(sc)    sc->host
#define sc_channel(sc) sc->channel
#define sc_target(sc)  sc->target
#define sc_lun(sc)     sc->lun
#define sc_flags(sc)   sc->flags
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)
#define pci_map_sg(pdev, sg_list, nseg, direction) (nseg)
#define pci_unmap_sg(pdev, sg_list, nseg, direction)
#define sg_dma_address(sg) (virt_to_bus((sg)->address))
#define sg_dma_len(sg) ((sg)->length)
#endif

extern spinlock_t driver_lock;
extern struct timer_list beeper_timer;
extern void stop_beeper(void);

/*entry.c*/
int HPTLIBAPI fOsBuildSgl(PCommand pCmd, FPSCAT_GATH pSg, int logical);
typedef void (*HPT_DPC)(void*,UCHAR);
int hpt_queue_dpc(HPT_DPC dpc, void *arg, UCHAR flags);
void hpt_rebuild_data_block(PVDevice pArray, UCHAR flags);

#define DUPLICATE      0
#define INITIALIZE     1
#define REBUILD_PARITY 2
#define VERIFY_R         3

#endif
